﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 로드시 처리하기 - Window_Loaded(sender, e)

        /// <summary>
        /// 윈도우 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            double width  = this.canvas.ActualWidth;
            double height = this.canvas.ActualHeight;

            double centerX = width / 2;
            double centerY = height / 2;

            for(int radius = 200; radius > 0; radius -= 50)
            {
                byte blue  = (byte)(radius / 2 + 155);
                byte red   = (byte)(blue / 2);
                byte green = (byte)(blue / 2);

                Ellipse ellipse = new Ellipse();

                ellipse.Fill            = new SolidColorBrush(Color.FromArgb(255, red, green, blue));
                ellipse.Stroke          = Brushes.Black;
                ellipse.StrokeThickness = 3;

                Canvas.SetLeft(ellipse, centerX - radius);
                Canvas.SetTop (ellipse, centerY - radius);

                ellipse.Width  = 2 * radius;
                ellipse.Height = 2 * radius;

                this.canvas.Children.Add(ellipse);
            }

            double fontSize = 10;

            for(int radius = 25; radius < 200; radius += 50)
            {
                const double ANGLE_COUNT = 6;

                double deltaTheta = 360 / ANGLE_COUNT;
                double theta      = deltaTheta / 2;

                for(int i = 0; i < ANGLE_COUNT; i++)
                {
                    double angle = 90 - theta;

                    double radians = (angle - 90) / 180 * Math.PI;

                    double x = centerX + radius * Math.Cos(radians);
                    double y = centerY + radius * Math.Sin(radians);

                    string text = ((int)theta).ToString();

                    DrawText
                    (
                        this.canvas,
                        text,
                        new Point(x, y),
                        angle,
                        fontSize,
                        HorizontalAlignment.Center,
                        VerticalAlignment.Center
                    );

                    theta += deltaTheta;
                }

                fontSize += 3;
            }
        }

        #endregion
        #region 슬라이더 값 변경시 처리하기 - slider_ValueChanged(sender, e)

        /// <summary>
        /// 슬라이더 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void slider_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if(!IsInitialized)
            {
                return;
            }

            this.zoomLabel.Content = slider.Value + "%";

            double scale = (double)(slider.Value / 100.0);

            this.canvas.LayoutTransform = new ScaleTransform(scale, scale);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 텍스트 그리기 - DrawText(canvas, text, location, angle, fontSize, horizontalAlignment, verticalAlignment)

        /// <summary>
        /// 텍스트 그리기
        /// </summary>
        /// <param name="canvas">캔버스</param>
        /// <param name="text">텍스트</param>
        /// <param name="location">위치</param>
        /// <param name="angle">각도</param>
        /// <param name="fontSize">폰트 크기</param>
        /// <param name="horizontalAlignment">수평 정렬</param>
        /// <param name="verticalAlignment">수직 정렬</param>
        private void DrawText(Canvas canvas, string text, Point location, double angle, double fontSize, HorizontalAlignment horizontalAlignment, VerticalAlignment verticalAlignment)
        {
            Label label = new Label();

            label.Content  = text;
            label.FontSize = fontSize;

            canvas.Children.Add(label);

            if(angle != 0)
            {
                label.LayoutTransform = new RotateTransform(angle);
            }

            label.Measure(new Size(double.MaxValue, double.MaxValue));

            double x = location.X;

            if(horizontalAlignment == HorizontalAlignment.Center)
            {
                x -= label.DesiredSize.Width / 2;
            }
            else if(horizontalAlignment == HorizontalAlignment.Right)
            {
                x -= label.DesiredSize.Width;
            }

            Canvas.SetLeft(label, x);

            double y = location.Y;

            if(verticalAlignment == VerticalAlignment.Center)
            {
                y -= label.DesiredSize.Height / 2;
            }
            else if(verticalAlignment == VerticalAlignment.Bottom)
            {
                y -= label.DesiredSize.Height;
            }

            Canvas.SetTop(label, y);
        }

        #endregion
    }
}